import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JPanel;

public class AnimationPanel2D extends JPanel{
	
	int N = 500;
    int L = 10;
    double u = 0.05;
    double etaVicsek = 0.1;
    double etaChate = 0;
    double radius = 0.4;
    double psi,psix,psiy,dens;

    double[] x = new double[N];
    double[] y = new double[N];
    double[] a = new double[N];
    Celda2D[][] C = new Celda2D[L][L];
    
    int spX,spY,oX,oY;
    Font font = new Font("SansSerif",Font.BOLD,20);
    Color color = new Color(255,150,150);
    
    // ====================================
    public AnimationPanel2D(int sx,int sy){

		// Parameters for the visual display 
		oX = 100;
		oY = 50;
		spX = 500;
		spY = 500;
		setSize(spX,spY);
	
		// initializes coordinates 
		psix = 0; 
		psiy = 0;
		for(int n = 0; n < N; ++n){
		    x[n] = L*Math.random();
		    y[n] = L*Math.random();
		    a[n] = 2*Math.PI*Math.random();
		    psix += Math.cos(a[n]);
		    psiy += Math.sin(a[n]);
		}
		psi = Math.sqrt((psix*psix)+(psiy*psiy));
		psi /= (1.0*N);
		dens = ((1.0*N)/(1.0*L*L))*Math.PI*radius*radius;

		// creates cells
		for(int lx = 0; lx < L; ++lx)
		    for(int ly = 0; ly < L; ++ly)
			C[lx][ly] = new Celda2D(lx,ly,L);
	
    } // end of constructor animationPanel


    // =====================================
    public void paintComponent(Graphics g){
	
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
	
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				    RenderingHints.VALUE_ANTIALIAS_ON);
	
		Ellipse2D circle = new Ellipse2D.Double();
		Line2D line = new Line2D.Double();
		Rectangle2D rectangle = new Rectangle2D.Double(oX,oY,spX,spY);
		
		double xc1,yc1,xc2,yc2,vx,vy;
		int r=3,ipsi,idens;
		String sPsi,sDens;
		
		g2.setPaint(Color.BLACK);
		g2.draw(rectangle);
	
		g2.setPaint(color);
		xc1 = oX + (spX/2);
		yc1 = oY + (spY/2);
		xc2 = xc1 + ((spX/L)*radius);
		yc2 = yc1 + ((spY/L)*radius);
		circle.setFrameFromCenter(xc1,yc1,xc2,yc2);
		g2.fill(circle);

		g2.setPaint(Color.BLACK);
		psix = 0;
		psiy = 0;
		for(int n = 0; n < N; ++n){
		    xc1 = oX + (spX*x[n])/(1.0*L);
		    yc1 = oY + (spY*y[n])/(1.0*L);
		    circle.setFrameFromCenter(xc1,yc1,xc1+r,yc1+r);
		    g2.fill(circle);
		    
		    vx = Math.cos(a[n]);
		    vy = Math.sin(a[n]);
		    psix += vx;
		    psiy += vy;
		    xc2 = xc1 - (10* vx);
		    yc2 = yc1 - (10* vy);
		    line.setLine(xc1,yc1,xc2,yc2);
		    g2.draw(line);
		}
		psix /= N;
		psiy /= N;
		psi = Math.sqrt((psix*psix) + (psiy*psiy));
		ipsi = (int)(10000*psi);
		psi = (1.0*ipsi)/10000;
		sPsi = "Psi = " + psi;
		idens = (int)(10000*dens);
		dens = (1.0*idens)/10000;
		sDens = "K = " + dens;
		
		g2.setPaint(Color.BLUE);
		g2.setFont(font);
		g2.drawString(sPsi,oX,40);
		g2.drawString(sDens, 450, 40);
	
    } // end of paintComponent

    // ============================
    public void startAnimation(){
		for(int t = 0; t <= 500000; ++t){
		    Methods2D.updateCeldas(C,x,y);
			dens = Methods2D.updateVicsekChate(C,x,y,a,radius,etaVicsek,etaChate,u);	
		    repaint();
		}
    }// end of startAnimation

    // ============================
    public void restartAnimation(){
		for(int n = 0; n < N; ++n){
		    x[n] = L*Math.random();
		    y[n] = L*Math.random();
		    a[n] = 2*Math.PI*Math.random();
		}
		dens = ((1.0*N)/(1.0*L*L))*Math.PI*radius*radius;
		repaint();
    }// end of startAnimation

    public double getVelocity(){
    	return u;
    }
    public double getNoiseVicsek(){
    	return etaVicsek;
    }
    public double getNoiseChate(){
    	return etaChate;
    }
    public double getRadius(){
		return radius;
	}
    public void setVelocity(double v){
    	u = v;
    }
    public void setNoiseVicsek(double n){
    	etaVicsek = n;
    	repaint();
    }
    public void setNoiseChate(double n){
    	etaChate = n;
    	repaint();
    }
    public void setRadius(double r){
    	radius = r;
    	repaint();
    }
}
