
public class Methods2D {


    // ============================
    public static void updateCeldas(Celda2D[][] C, double[] x, double[] y){
    	int L = C.length;
    	int N = x.length;
		int[][] nC = new int[L][L]; // numero de particulas en cada Celda2D
		int n,ix,iy;
		
		for(ix = 0; ix < L; ++ix){
		    for(iy = 0; iy < L; ++iy){
			nC[ix][iy] = 0;
		    }
		}
		
		// encuentra el numero de particulas en cada Celda2D
		for(n = 0; n < N; ++n){
		    ix = (int)x[n];
		    iy = (int)y[n];
		    ++nC[ix][iy];
		}
		// define los arreglos de cada Celda2D para guardar
		// las particulas que se encuentran en dicha Celda2D
		for(ix = 0; ix < L; ++ix){
		    for(iy = 0; iy < L; ++iy){
				C[ix][iy].setNP(nC[ix][iy]);
				nC[ix][iy] = 0;
		    }
		}
		// llena los arreglos de cada Celda2D con los indices
		// de las particulas que se encuentran en dicha Celda2D
		for(n = 0; n < N; ++n){
		    ix = (int)x[n];
		    iy = (int)y[n];
		    C[ix][iy].setPar(nC[ix][iy],n);
		    ++nC[ix][iy];
		}
    } // Fin de updateCeldas
    
    //  ================================
    public static double updateVicsek(Celda2D[][] C, double[] x, double[] y, double[] a, 
    		double radius, double eta,double u){
    	int L = C.length;
    	int N = x.length;
    	double[] angle = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv;
    	double dx,dy,vx,vy,dr;
    	double psi,psiX,psiY;
	

    	// ciclos sobre las Celda2Ds
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx = 0;
    				vy = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx <= radius) && (dy <= radius)){
    						if(dr <= radius){
    							vx += Math.cos(a[ipv]);
    							vy += Math.sin(a[ipv]);
    						}
    					}
    				}
    				angle[ip] = Math.atan2(vy,vx);
    			}
    		}
    	}
    	psi = 0;
    	psiX = 0;
    	psiY = 0;
    	for(n = 0; n < N; ++n){
    		a[n] = angle[n] + ((2*Math.PI*eta)*(Math.random() - 0.5));
    		vx = u*Math.cos(a[n]);
    		vy = u*Math.sin(a[n]);
    		psiX += vx;
    		psiY += vy;
    		x[n] += vx;
    		y[n] += vy;
    		if(x[n] < 0)
    			x[n] += L;
    		if(x[n] >= L)
    			x[n] -= L;
    		if(y[n] < 0)
    			y[n] += L;
    		if(y[n] >= L)
    			y[n] -= L;
    		
    	}
    	psi = Math.sqrt((psiX*psiX)+(psiY*psiY));
    	psi /= (u*N);
    	return psi;
    }// Fin de updateVicsek

    // =============================
    public static double updateChate(Celda2D[][] C,double[] x,double[] y, double[] a,
    		double radius,double eta, double u)
    {
    	int L = C.length;
    	int N = x.length;
    	double[] vx = new double[N];
    	double[] vy = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv,nvecinos;
    	double dx,dy,da,dr,psiX,psiY;
    	double psi;
    	
    	// ciclos sobre las Celda2Ds
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx[ip] = 0;
    				vy[ip] = 0;
    				nvecinos = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// 	ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx <= radius) && (dy <= radius)){
    						if(dr <= radius){
    							vx[ip] += Math.cos(a[ipv]);
    							vy[ip] += Math.sin(a[ipv]);
    							++nvecinos;
    						}
    					}
    				}
    				vx[ip] = vx[ip]/nvecinos;
    				vy[ip] = vy[ip]/nvecinos;
    			}
    		}
    	}
    	psiX = 0;
    	psiY = 0;
    	for(n = 0; n < N; ++n){
    		da = 2*Math.PI*Math.random();
    		dx = vx[n] + eta*Math.cos(da);
    		dy = vy[n] + eta*Math.sin(da);
    		
    		a[n] = Math.atan2(dy,dx);
    		dx = u*Math.cos(a[n]);
    		dy = u*Math.sin(a[n]);
    		psiX += dx;
    		psiY += dy;
    		
    		x[n] += dx;
    		y[n] += dy;
    		if(x[n] < 0)
    			x[n] += L;
    		if(x[n] >= L)
    			x[n] -= L;
    		if(y[n] < 0)
    			y[n] += L;
    		if(y[n] >= L)
    			y[n] -= L;
    	}
    	psi = Math.sqrt((psiX*psiX)+(psiY*psiY));
    	psi /= (u*N);
    	return psi;
    }// Fin de updateChate

    //  =============================
    public static double updateVicsekChate(Celda2D[][] C,double[] x,double[] y, double[] a,
    		double radius,double eta1, double eta2,double u)
    {
    	int L = C.length;
    	int N = x.length;
    	double[] vx = new double[N];
    	double[] vy = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv,nvecinos;
    	double dx,dy,da,dr,dens;
    	
    	// ciclos sobre las Celda2Ds
    	dens = 0;
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx[ip] = 0;
    				vy[ip] = 0;
    				nvecinos = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// 	ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx < radius) && (dy < radius)){
    						if(dr < radius){
    							vx[ip] += Math.cos(a[ipv]);
    							vy[ip] += Math.sin(a[ipv]);
    							++nvecinos;
    						}
    					}
    				}
    				vx[ip] = vx[ip]/nvecinos;
    				vy[ip] = vy[ip]/nvecinos;
    				dens += nvecinos;
    			}
    		}
    	}
    	dens /= N;
    	for(n = 0; n < N; ++n){
    		da = 2*Math.PI*Math.random();
    		dx = vx[n] + eta2*Math.cos(da);
    		dy = vy[n] + eta2*Math.sin(da);
    		
    		a[n] = Math.atan2(dy,dx);
    		a[n] += (2*Math.PI*eta1*(Math.random()-0.5));
    		dx = u*Math.cos(a[n]);
    		dy = u*Math.sin(a[n]);
    		
    		x[n] += dx;
    		y[n] += dy;
    		if(x[n] < 0)
    			x[n] += L;
    		if(x[n] >= L)
    			x[n] -= L;
    		if(y[n] < 0)
    			y[n] += L;
    		if(y[n] >= L)
    			y[n] -= L;
    	}
    	return dens;
    }// Fin de updateVicsekChate
    
    //  ================================
    public static double updateVicsekRandomMixing(Celda2D[][] C, double[] x, double[] y, double[] a, 
    		double radius, double eta){
    	int L = C.length;
    	int N = x.length;
    	double[] angle = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv;
    	double dx,dy,vx,vy,dr;
    	double psi,psiX,psiY;
	

    	// ciclos sobre las Celda2Ds
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx = 0;
    				vy = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx <= radius) && (dy <= radius)){
    						if(dr <= radius){
    							vx += Math.cos(a[ipv]);
    							vy += Math.sin(a[ipv]);
    						}
    					}
    				}
    				angle[ip] = Math.atan2(vy,vx);
    			}
    		}
    	}
    	psi = 0;
    	psiX = 0;
    	psiY = 0;
    	for(n = 0; n < N; ++n){
    		a[n] = angle[n] + ((2*Math.PI*eta)*(Math.random() - 0.5));
    		vx = Math.cos(a[n]);
    		vy = Math.sin(a[n]);
    		psiX += vx;
    		psiY += vy;
    		x[n] = L*Math.random();
    		y[n] = L*Math.random();
    		
    	}
    	psi = Math.sqrt((psiX*psiX)+(psiY*psiY));
    	psi /= (1.0*N);
    	return psi;
    }// Fin de updateVicsek

    // =============================
    public static double updateChateRandomMixing(Celda2D[][] C,double[] x,double[] y, double[] a,
    		double radius,double eta)
    {
    	int L = C.length;
    	int N = x.length;
    	double[] vx = new double[N];
    	double[] vy = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv,nvecinos;
    	double dx,dy,da,dr,psiX,psiY;
    	double psi;
    	
    	// ciclos sobre las Celda2Ds
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx[ip] = 0;
    				vy[ip] = 0;
    				nvecinos = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// 	ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx <= radius) && (dy <= radius)){
    						if(dr <= radius){
    							vx[ip] += Math.cos(a[ipv]);
    							vy[ip] += Math.sin(a[ipv]);
    							++nvecinos;
    						}
    					}
    				}
    				vx[ip] = vx[ip]/nvecinos;
    				vy[ip] = vy[ip]/nvecinos;
    			}
    		}
    	}
    	psiX = 0;
    	psiY = 0;
    	for(n = 0; n < N; ++n){
    		da = 2*Math.PI*Math.random();
    		dx = vx[n] + eta*Math.cos(da);
    		dy = vy[n] + eta*Math.sin(da);
    		
    		a[n] = Math.atan2(dy,dx);
    		dx = Math.cos(a[n]);
    		dy = Math.sin(a[n]);
    		psiX += dx;
    		psiY += dy;
    		
    		x[n] = L*Math.random();
    		y[n] = L*Math.random();
    	}
    	psi = Math.sqrt((psiX*psiX)+(psiY*psiY));
    	psi /= (1.0*N);
    	return psi;
    }// Fin de updateChate

    //  =============================
    public static double updateVicsekChateRandomMixing(Celda2D[][] C,double[] x,double[] y, double[] a,
    		double radius,double eta1, double eta2)
    {
    	int L = C.length;
    	int N = x.length;
    	double[] vx = new double[N];
    	double[] vy = new double[N];
    	int lx,ly,lxv,lyv,cx,cy;
    	int m,n,nv,ip,ipv,nvecinos;
    	double dx,dy,da,dr,dens;
    	
    	// ciclos sobre las Celda2Ds
    	dens = 0;
    	for(lx = 0; lx < L; ++lx){
    		for(ly = 0; ly < L; ++ly){
    			// ciclo sobre las particulas de cada Celda2D
    			for(n = 0; n < C[lx][ly].getNP(); ++n){
    				ip = C[lx][ly].getPar(n);
    				vx[ip] = 0;
    				vy[ip] = 0;
    				nvecinos = 0;
    				// ciclo sobre las Celda2Ds vecinas
    				for(m = 0; m < 9; ++m){
    					lxv = C[lx][ly].getVecinoX(m);
    					lyv = C[lx][ly].getVecinoY(m);
    					cx = C[lx][ly].getPhantomCompX(m);
    					cy = C[lx][ly].getPhantomCompY(m);
    					// 	ciclo sobre las particulas de la Celda2D vecina
    					for(nv = 0; nv < C[lxv][lyv].getNP(); ++nv){
    						ipv = C[lxv][lyv].getPar(nv);
    						dx = Math.abs(x[ip]-x[ipv]-cx);
    						dy = Math.abs(y[ip]-y[ipv]-cy);
    						dr = Math.sqrt((dx*dx)+(dy*dy));
    						//if((dx < radius) && (dy < radius)){
    						if(dr < radius){
    							vx[ip] += Math.cos(a[ipv]);
    							vy[ip] += Math.sin(a[ipv]);
    							++nvecinos;
    						}
    					}
    				}
    				vx[ip] = vx[ip]/nvecinos;
    				vy[ip] = vy[ip]/nvecinos;
    				dens += nvecinos;
    			}
    		}
    	}
    	dens /= N;
    	for(n = 0; n < N; ++n){
    		da = 2*Math.PI*Math.random();
    		dx = vx[n] + eta2*Math.cos(da);
    		dy = vy[n] + eta2*Math.sin(da);
    		
    		a[n] = Math.atan2(dy,dx);
    		a[n] += (2*Math.PI*eta1*(Math.random()-0.5));
    		dx = Math.cos(a[n]);
    		dy = Math.sin(a[n]);
    		
    		x[n] = L*Math.random();
    		y[n] = L*Math.random();
    	}
    	return dens;
    }// Fin de updateVicsekChate
}
